@@ -15,39 +15,43 @@
 
 """Identity v3 Role action implementations"""
 
-import six
-import sys
+import logging
 
 from keystoneauth1 import exceptions as ks_exc
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+import six
 
-from openstackclient.common import command
-from openstackclient.common import utils
-from openstackclient.i18n import _  # noqa
+from openstackclient.i18n import _
 from openstackclient.identity import common
 
 
+LOG = logging.getLogger(__name__)
+
+
 def _add_identity_and_resource_options_to_parser(parser):
     domain_or_project = parser.add_mutually_exclusive_group()
     domain_or_project.add_argument(
         '--domain',
         metavar='<domain>',
-        help='Include <domain> (name or ID)',
+        help=_('Include <domain> (name or ID)'),
     )
     domain_or_project.add_argument(
         '--project',
         metavar='<project>',
-        help='Include <project> (name or ID)',
+        help=_('Include <project> (name or ID)'),
     )
     user_or_group = parser.add_mutually_exclusive_group()
     user_or_group.add_argument(
         '--user',
         metavar='<user>',
-        help='Include <user> (name or ID)',
+        help=_('Include <user> (name or ID)'),
     )
     user_or_group.add_argument(
         '--group',
         metavar='<group>',
-        help='Include <group> (name or ID)',
+        help=_('Include <group> (name or ID)'),
     )
     common.add_group_domain_option_to_parser(parser)
     common.add_project_domain_option_to_parser(parser)
@@ -105,49 +109,59 @@ def _process_identity_and_resource_options(parsed_args,
 
 
 class AddRole(command.Command):
-    """Adds a role to a user or group on a domain or project"""
+    _description = _("Adds a role assignment to a user or group on a domain "
+                     "or project")
 
     def get_parser(self, prog_name):
         parser = super(AddRole, self).get_parser(prog_name)
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role to add to <user> (name or ID)',
+            help=_('Role to add to <user> (name or ID)'),
         )
         _add_identity_and_resource_options_to_parser(parser)
+        common.add_role_domain_option_to_parser(parser)
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        if (not parsed_args.user and not parsed_args.domain
-                and not parsed_args.group and not parsed_args.project):
-            return
+        if (not parsed_args.user and not parsed_args.domain and
+                not parsed_args.group and not parsed_args.project):
+            msg = _("Role not added, incorrect set of arguments "
+                    "provided. See openstack --help for more details")
+            raise exceptions.CommandError(msg)
+
+        domain_id = None
+        if parsed_args.role_domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.role_domain).id
         role = utils.find_resource(
             identity_client.roles,
             parsed_args.role,
+            domain_id=domain_id
         )
 
         kwargs = _process_identity_and_resource_options(
             parsed_args, self.app.client_manager.identity)
-        if not kwargs:
-            sys.stderr.write("Role not added, incorrect set of arguments "
-                             "provided. See openstack --help for more "
-                             "details\n")
-            return
 
         identity_client.roles.grant(role.id, **kwargs)
 
 
 class CreateRole(command.ShowOne):
-    """Create new role"""
+    _description = _("Create new role")
 
     def get_parser(self, prog_name):
         parser = super(CreateRole, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<role-name>',
-            help='New role name',
+            help=_('New role name'),
+        )
+        parser.add_argument(
+            '--domain',
+            metavar='<domain>',
+            help=_('Domain the role belongs to (name or ID)'),
         )
         parser.add_argument(
             '--or-show',
@@ -159,22 +173,30 @@ class CreateRole(command.ShowOne):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
+        domain_id = None
+        if parsed_args.domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.domain).id
+
         try:
-            role = identity_client.roles.create(name=parsed_args.name)
-        except ks_exc.Conflict as e:
+            role = identity_client.roles.create(
+                name=parsed_args.name, domain=domain_id)
+
+        except ks_exc.Conflict:
             if parsed_args.or_show:
                 role = utils.find_resource(identity_client.roles,
-                                           parsed_args.name)
-                self.log.info('Returning existing role %s', role.name)
+                                           parsed_args.name,
+                                           domain_id=domain_id)
+                LOG.info(_('Returning existing role %s'), role.name)
             else:
-                raise e
+                raise
 
         role._info.pop('links')
         return zip(*sorted(six.iteritems(role._info)))
 
 
 class DeleteRole(command.Command):
-    """Delete role(s)"""
+    _description = _("Delete role(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteRole, self).get_parser(prog_name)
@@ -182,26 +204,61 @@ class DeleteRole(command.Command):
             'roles',
             metavar='<role>',
             nargs="+",
-            help='Role(s) to delete (name or ID)',
+            help=_('Role(s) to delete (name or ID)'),
+        )
+        parser.add_argument(
+            '--domain',
+            metavar='<domain>',
+            help=_('Domain the role belongs to (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
+        domain_id = None
+        if parsed_args.domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.domain).id
+        errors = 0
         for role in parsed_args.roles:
-            role_obj = utils.find_resource(
-                identity_client.roles,
-                role,
-            )
-            identity_client.roles.delete(role_obj.id)
+            try:
+                role_obj = utils.find_resource(
+                    identity_client.roles,
+                    role,
+                    domain_id=domain_id
+                )
+                identity_client.roles.delete(role_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete role with "
+                          "name or ID '%(role)s': %(e)s"),
+                          {'role': role, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.roles)
+            msg = (_("%(errors)s of %(total)s roles failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListRole(command.Lister):
-    """List roles"""
+    _description = _("List roles")
 
     def get_parser(self, prog_name):
         parser = super(ListRole, self).get_parser(prog_name)
+
+        # TODO(henry-nash): The use of the List Role command to list
+        # assignments (as well as roles) has been deprecated. In order
+        # to support domain specific roles, we are overriding the domain
+        # option to allow specification of the domain for the role. This does
+        # not conflict with any existing commands, since for the deprecated
+        # assignments listing you were never allowed to only specify a domain
+        # (you also needed to specify a user).
+        #
+        # Once we have removed the deprecated options entirely, we must
+        # replace the call to _add_identity_and_resource_options_to_parser()
+        # below with just adding the domain option into the parser.
         _add_identity_and_resource_options_to_parser(parser)
         return parser
 
@@ -235,8 +292,14 @@ class ListRole(command.Lister):
 
         # no user or group specified, list all roles in the system
         if not parsed_args.user and not parsed_args.group:
-            columns = ('ID', 'Name')
-            data = identity_client.roles.list()
+            if not parsed_args.domain:
+                columns = ('ID', 'Name')
+                data = identity_client.roles.list()
+            else:
+                columns = ('ID', 'Name', 'Domain')
+                data = identity_client.roles.list(domain_id=domain.id)
+                for role in data:
+                    role.domain = domain.name
         elif parsed_args.user and parsed_args.domain:
             columns = ('ID', 'Name', 'Domain', 'User')
             data = identity_client.roles.list(
@@ -247,6 +310,10 @@ class ListRole(command.Lister):
             for user_role in data:
                 user_role.user = user.name
                 user_role.domain = domain.name
+            self.log.warning(_('Listing assignments using role list is '
+                               'deprecated. Use role assignment list --user '
+                               '<user-name> --domain <domain-name> --names '
+                               'instead.'))
         elif parsed_args.user and parsed_args.project:
             columns = ('ID', 'Name', 'Project', 'User')
             data = identity_client.roles.list(
@@ -257,6 +324,10 @@ class ListRole(command.Lister):
             for user_role in data:
                 user_role.user = user.name
                 user_role.project = project.name
+            self.log.warning(_('Listing assignments using role list is '
+                               'deprecated. Use role assignment list --user '
+                               '<user-name> --project <project-name> --names '
+                               'instead.'))
         elif parsed_args.user:
             columns = ('ID', 'Name')
             data = identity_client.roles.list(
@@ -264,6 +335,10 @@ class ListRole(command.Lister):
                 domain='default',
                 os_inherit_extension_inherited=parsed_args.inherited
             )
+            self.log.warning(_('Listing assignments using role list is '
+                               'deprecated. Use role assignment list --user '
+                               '<user-name> --domain default --names '
+                               'instead.'))
         elif parsed_args.group and parsed_args.domain:
             columns = ('ID', 'Name', 'Domain', 'Group')
             data = identity_client.roles.list(
@@ -274,6 +349,10 @@ class ListRole(command.Lister):
             for group_role in data:
                 group_role.group = group.name
                 group_role.domain = domain.name
+            self.log.warning(_('Listing assignments using role list is '
+                               'deprecated. Use role assignment list --group '
+                               '<group-name> --domain <domain-name> --names '
+                               'instead.'))
         elif parsed_args.group and parsed_args.project:
             columns = ('ID', 'Name', 'Project', 'Group')
             data = identity_client.roles.list(
@@ -284,11 +363,15 @@ class ListRole(command.Lister):
             for group_role in data:
                 group_role.group = group.name
                 group_role.project = project.name
+            self.log.warning(_('Listing assignments using role list is '
+                               'deprecated. Use role assignment list --group '
+                               '<group-name> --project <project-name> --names '
+                               'instead.'))
         else:
-            sys.stderr.write("Error: If a user or group is specified, either "
-                             "--domain or --project must also be specified to "
-                             "list role grants.\n")
-            return ([], [])
+            msg = _("Error: If a user or group is specified, "
+                    "either --domain or --project must also be "
+                    "specified to list role grants.")
+            raise exceptions.CommandError(msg)
 
         return (columns,
                 (utils.get_item_properties(
@@ -298,93 +381,110 @@ class ListRole(command.Lister):
 
 
 class RemoveRole(command.Command):
-    """Remove role from domain/project : user/group"""
+    _description = _("Removes a role assignment from domain/project : "
+                     "user/group")
 
     def get_parser(self, prog_name):
         parser = super(RemoveRole, self).get_parser(prog_name)
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role to remove (name or ID)',
+            help=_('Role to remove (name or ID)'),
         )
         _add_identity_and_resource_options_to_parser(parser)
+        common.add_role_domain_option_to_parser(parser)
+
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        if (not parsed_args.user and not parsed_args.domain
-                and not parsed_args.group and not parsed_args.project):
-            sys.stderr.write("Incorrect set of arguments "
-                             "provided. See openstack --help for more "
-                             "details\n")
-            return
+        if (not parsed_args.user and not parsed_args.domain and
+                not parsed_args.group and not parsed_args.project):
+            msg = _("Incorrect set of arguments provided. "
+                    "See openstack --help for more details")
+            raise exceptions.CommandError(msg)
+
+        domain_id = None
+        if parsed_args.role_domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.role_domain).id
         role = utils.find_resource(
             identity_client.roles,
             parsed_args.role,
+            domain_id=domain_id
         )
 
         kwargs = _process_identity_and_resource_options(
             parsed_args, self.app.client_manager.identity)
-        if not kwargs:
-            sys.stderr.write("Role not removed, incorrect set of arguments \
-            provided. See openstack --help for more details\n")
-            return
         identity_client.roles.revoke(role.id, **kwargs)
 
 
 class SetRole(command.Command):
-    """Set role properties"""
+    _description = _("Set role properties")
 
     def get_parser(self, prog_name):
         parser = super(SetRole, self).get_parser(prog_name)
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role to modify (name or ID)',
+            help=_('Role to modify (name or ID)'),
+        )
+        parser.add_argument(
+            '--domain',
+            metavar='<domain>',
+            help=_('Domain the role belongs to (name or ID)'),
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
-            help='Set role name',
+            help=_('Set role name'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        if not parsed_args.name:
-            sys.stderr.write("Incorrect set of arguments "
-                             "provided. See openstack --help for more "
-                             "details\n")
-            return
-        role = utils.find_resource(
-            identity_client.roles,
-            parsed_args.role,
-        )
+        domain_id = None
+        if parsed_args.domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.domain).id
+
+        role = utils.find_resource(identity_client.roles,
+                                   parsed_args.role,
+                                   domain_id=domain_id)
 
         identity_client.roles.update(role.id, name=parsed_args.name)
 
 
 class ShowRole(command.ShowOne):
-    """Display role details"""
+    _description = _("Display role details")
 
     def get_parser(self, prog_name):
         parser = super(ShowRole, self).get_parser(prog_name)
         parser.add_argument(
             'role',
             metavar='<role>',
-            help='Role to display (name or ID)',
+            help=_('Role to display (name or ID)'),
+        )
+        parser.add_argument(
+            '--domain',
+            metavar='<domain>',
+            help=_('Domain the role belongs to (name or ID)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        role = utils.find_resource(
-            identity_client.roles,
-            parsed_args.role,
-        )
+        domain_id = None
+        if parsed_args.domain:
+            domain_id = common.find_domain(identity_client,
+                                           parsed_args.domain).id
+
+        role = utils.find_resource(identity_client.roles,
+                                   parsed_args.role,
+                                   domain_id=domain_id)
 
         role._info.pop('links')
         return zip(*sorted(six.iteritems(role._info)))
