@@ -13,41 +13,47 @@
 
 """Service Provider action implementations"""
 
-import six
-import sys
+import logging
 
-from openstackclient.common import command
-from openstackclient.common import utils
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+import six
+
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
 
 
 class CreateServiceProvider(command.ShowOne):
-    """Create new service provider"""
+    _description = _("Create new service provider")
 
     def get_parser(self, prog_name):
         parser = super(CreateServiceProvider, self).get_parser(prog_name)
         parser.add_argument(
             'service_provider_id',
             metavar='<name>',
-            help='New service provider name (must be unique)'
+            help=_('New service provider name (must be unique)'),
         )
         parser.add_argument(
             '--auth-url',
             metavar='<auth-url>',
             required=True,
-            help='Authentication URL of remote federated service provider '
-                 '(required)',
+            help=_('Authentication URL of remote federated service provider '
+                   '(required)'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New service provider description',
+            help=_('New service provider description'),
         )
         parser.add_argument(
             '--service-provider-url',
             metavar='<sp-url>',
             required=True,
-            help='A service URL where SAML assertions are being sent '
-                 '(required)',
+            help=_('A service URL where SAML assertions are being sent '
+                   '(required)'),
         )
 
         enable_service_provider = parser.add_mutually_exclusive_group()
@@ -56,13 +62,13 @@ class CreateServiceProvider(command.ShowOne):
             dest='enabled',
             action='store_true',
             default=True,
-            help='Enable the service provider (default)',
+            help=_('Enable the service provider (default)'),
         )
         enable_service_provider.add_argument(
             '--disable',
             dest='enabled',
             action='store_false',
-            help='Disable the service provider',
+            help=_('Disable the service provider'),
         )
 
         return parser
@@ -81,25 +87,39 @@ class CreateServiceProvider(command.ShowOne):
 
 
 class DeleteServiceProvider(command.Command):
-    """Delete service provider"""
+    _description = _("Delete service provider(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteServiceProvider, self).get_parser(prog_name)
         parser.add_argument(
             'service_provider',
             metavar='<service-provider>',
-            help='Service provider to delete',
+            nargs='+',
+            help=_('Service provider(s) to delete'),
         )
         return parser
 
     def take_action(self, parsed_args):
         service_client = self.app.client_manager.identity
-        service_client.federation.service_providers.delete(
-            parsed_args.service_provider)
+        result = 0
+        for i in parsed_args.service_provider:
+            try:
+                service_client.federation.service_providers.delete(i)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete service provider with "
+                          "name or ID '%(provider)s': %(e)s"),
+                          {'provider': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.service_provider)
+            msg = (_("%(result)s of %(total)s service providers failed"
+                   " to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListServiceProvider(command.Lister):
-    """List service providers"""
+    _description = _("List service providers")
 
     def take_action(self, parsed_args):
         service_client = self.app.client_manager.identity
@@ -114,41 +134,42 @@ class ListServiceProvider(command.Lister):
 
 
 class SetServiceProvider(command.Command):
-    """Set service provider properties"""
+    _description = _("Set service provider properties")
 
     def get_parser(self, prog_name):
         parser = super(SetServiceProvider, self).get_parser(prog_name)
         parser.add_argument(
             'service_provider',
             metavar='<service-provider>',
-            help='Service provider to modify',
+            help=_('Service provider to modify'),
         )
         parser.add_argument(
             '--auth-url',
             metavar='<auth-url>',
-            help='New Authentication URL of remote federated service provider',
+            help=_('New Authentication URL of remote '
+                   'federated service provider'),
         )
 
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New service provider description',
+            help=_('New service provider description'),
         )
         parser.add_argument(
             '--service-provider-url',
             metavar='<sp-url>',
-            help='New service provider URL, where SAML assertions are sent',
+            help=_('New service provider URL, where SAML assertions are sent'),
         )
         enable_service_provider = parser.add_mutually_exclusive_group()
         enable_service_provider.add_argument(
             '--enable',
             action='store_true',
-            help='Enable the service provider',
+            help=_('Enable the service provider'),
         )
         enable_service_provider.add_argument(
             '--disable',
             action='store_true',
-            help='Disable the service provider',
+            help=_('Disable the service provider'),
         )
         return parser
 
@@ -161,30 +182,24 @@ class SetServiceProvider(command.Command):
         elif parsed_args.disable is True:
             enabled = False
 
-        if not any((enabled is not None, parsed_args.description,
-                    parsed_args.service_provider_url,
-                    parsed_args.auth_url)):
-            sys.stdout.write("Service Provider not updated, no arguments "
-                             "present")
-            return (None, None)
-
-        service_provider = federation_client.service_providers.update(
-            parsed_args.service_provider, enabled=enabled,
+        federation_client.service_providers.update(
+            parsed_args.service_provider,
+            enabled=enabled,
             description=parsed_args.description,
             auth_url=parsed_args.auth_url,
-            sp_url=parsed_args.service_provider_url)
-        return zip(*sorted(six.iteritems(service_provider._info)))
+            sp_url=parsed_args.service_provider_url,
+        )
 
 
 class ShowServiceProvider(command.ShowOne):
-    """Display service provider details"""
+    _description = _("Display service provider details")
 
     def get_parser(self, prog_name):
         parser = super(ShowServiceProvider, self).get_parser(prog_name)
         parser.add_argument(
             'service_provider',
             metavar='<service-provider>',
-            help='Service provider to display',
+            help=_('Service provider to display'),
         )
         return parser
 
