@@ -28,62 +28,60 @@ class NetworkQosRuleTestsMinimumBandwidth(common.NetworkTests):
         if not self.haz_network:
             self.skipTest("No Network service present")
 
-        self.QOS_POLICY_NAME = 'qos_policy_' + uuid.uuid4().hex
+        self.QOS_POLICY_NAME = 'qos_policy_%s' % uuid.uuid4().hex
 
         self.openstack(
-            'network qos policy create ' +
-            self.QOS_POLICY_NAME
+            'network qos policy create %s' % self.QOS_POLICY_NAME
         )
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + self.QOS_POLICY_NAME)
+                        'network qos policy delete %s' % self.QOS_POLICY_NAME)
         cmd_output = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type minimum-bandwidth ' +
-            '--min-kbps 2800 ' +
-            '--egress ' +
+            'network qos rule create -f json '
+            '--type minimum-bandwidth '
+            '--min-kbps 2800 '
+            '--egress %s' %
             self.QOS_POLICY_NAME
         ))
         self.RULE_ID = cmd_output['id']
         self.addCleanup(self.openstack,
-                        'network qos rule delete ' +
-                        self.QOS_POLICY_NAME + ' ' +
-                        self.RULE_ID)
+                        'network qos rule delete %s %s' %
+                        (self.QOS_POLICY_NAME, self.RULE_ID))
         self.assertTrue(self.RULE_ID)
 
     def test_qos_rule_create_delete(self):
         # This is to check the output of qos rule delete
         policy_name = uuid.uuid4().hex
-        self.openstack('network qos policy create -f json ' + policy_name)
+        self.openstack('network qos policy create -f json %s' % policy_name)
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + policy_name)
+                        'network qos policy delete %s' % policy_name)
         rule = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type minimum-bandwidth ' +
-            '--min-kbps 2800 ' +
-            '--egress ' + policy_name
+            'network qos rule create -f json '
+            '--type minimum-bandwidth '
+            '--min-kbps 2800 '
+            '--egress %s' % policy_name
         ))
         raw_output = self.openstack(
-            'network qos rule delete ' +
-            policy_name + ' ' + rule['id'])
+            'network qos rule delete %s %s' %
+            (policy_name, rule['id']))
         self.assertEqual('', raw_output)
 
     def test_qos_rule_list(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule list -f json ' + self.QOS_POLICY_NAME))
+            'network qos rule list -f json %s' % self.QOS_POLICY_NAME))
         self.assertIn(self.RULE_ID, [rule['ID'] for rule in cmd_output])
 
     def test_qos_rule_show(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(self.RULE_ID, cmd_output['id'])
 
     def test_qos_rule_set(self):
-        self.openstack('network qos rule set --min-kbps 7500 ' +
-                       self.QOS_POLICY_NAME + ' ' + self.RULE_ID)
+        self.openstack('network qos rule set --min-kbps 7500 %s %s' %
+                       (self.QOS_POLICY_NAME, self.RULE_ID))
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(7500, cmd_output['min_kbps'])
 
 
@@ -96,58 +94,57 @@ class NetworkQosRuleTestsDSCPMarking(common.NetworkTests):
         if not self.haz_network:
             self.skipTest("No Network service present")
 
-        self.QOS_POLICY_NAME = 'qos_policy_' + uuid.uuid4().hex
+        self.QOS_POLICY_NAME = 'qos_policy_%s' % uuid.uuid4().hex
         self.openstack(
-            'network qos policy create ' +
-            self.QOS_POLICY_NAME
+            'network qos policy create %s' % self.QOS_POLICY_NAME
         )
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + self.QOS_POLICY_NAME)
+                        'network qos policy delete %s' % self.QOS_POLICY_NAME)
         cmd_output = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type dscp-marking ' +
-            '--dscp-mark 8 ' +
+            'network qos rule create -f json '
+            '--type dscp-marking '
+            '--dscp-mark 8 %s' %
             self.QOS_POLICY_NAME
         ))
         self.RULE_ID = cmd_output['id']
         self.addCleanup(self.openstack,
-                        'network qos rule delete ' +
-                        self.QOS_POLICY_NAME + ' ' + self.RULE_ID)
+                        'network qos rule delete %s %s' %
+                        (self.QOS_POLICY_NAME, self.RULE_ID))
         self.assertTrue(self.RULE_ID)
 
     def test_qos_rule_create_delete(self):
         # This is to check the output of qos rule delete
         policy_name = uuid.uuid4().hex
-        self.openstack('network qos policy create -f json ' + policy_name)
+        self.openstack('network qos policy create -f json %s' % policy_name)
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + policy_name)
+                        'network qos policy delete %s' % policy_name)
         rule = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type dscp-marking ' +
-            '--dscp-mark 8 ' + policy_name
+            'network qos rule create -f json '
+            '--type dscp-marking '
+            '--dscp-mark 8 %s' % policy_name
         ))
         raw_output = self.openstack(
-            'network qos rule delete ' +
-            policy_name + ' ' + rule['id'])
+            'network qos rule delete %s %s' %
+            (policy_name, rule['id']))
         self.assertEqual('', raw_output)
 
     def test_qos_rule_list(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule list -f json ' + self.QOS_POLICY_NAME))
+            'network qos rule list -f json %s' % self.QOS_POLICY_NAME))
         self.assertIn(self.RULE_ID, [rule['ID'] for rule in cmd_output])
 
     def test_qos_rule_show(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(self.RULE_ID, cmd_output['id'])
 
     def test_qos_rule_set(self):
-        self.openstack('network qos rule set --dscp-mark 32 ' +
-                       self.QOS_POLICY_NAME + ' ' + self.RULE_ID)
+        self.openstack('network qos rule set --dscp-mark 32 %s %s' %
+                       (self.QOS_POLICY_NAME, self.RULE_ID))
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(32, cmd_output['dscp_mark'])
 
 
@@ -160,65 +157,64 @@ class NetworkQosRuleTestsBandwidthLimit(common.NetworkTests):
         if not self.haz_network:
             self.skipTest("No Network service present")
 
-        self.QOS_POLICY_NAME = 'qos_policy_' + uuid.uuid4().hex
+        self.QOS_POLICY_NAME = 'qos_policy_%s' % uuid.uuid4().hex
         self.openstack(
-            'network qos policy create ' +
-            self.QOS_POLICY_NAME
+            'network qos policy create %s' % self.QOS_POLICY_NAME
         )
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + self.QOS_POLICY_NAME)
+                        'network qos policy delete %s' % self.QOS_POLICY_NAME)
         cmd_output = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type bandwidth-limit ' +
-            '--max-kbps 10000 ' +
-            '--max-burst-kbits 1400 ' +
-            '--egress ' +
+            'network qos rule create -f json '
+            '--type bandwidth-limit '
+            '--max-kbps 10000 '
+            '--max-burst-kbits 1400 '
+            '--egress %s' %
             self.QOS_POLICY_NAME
         ))
         self.RULE_ID = cmd_output['id']
         self.addCleanup(self.openstack,
-                        'network qos rule delete ' +
-                        self.QOS_POLICY_NAME + ' ' + self.RULE_ID)
+                        'network qos rule delete %s %s' %
+                        (self.QOS_POLICY_NAME, self.RULE_ID))
         self.assertTrue(self.RULE_ID)
 
     def test_qos_rule_create_delete(self):
         # This is to check the output of qos rule delete
         policy_name = uuid.uuid4().hex
-        self.openstack('network qos policy create -f json ' + policy_name)
+        self.openstack('network qos policy create -f json %s' % policy_name)
         self.addCleanup(self.openstack,
-                        'network qos policy delete ' + policy_name)
+                        'network qos policy delete %s' % policy_name)
         rule = json.loads(self.openstack(
-            'network qos rule create -f json ' +
-            '--type bandwidth-limit ' +
-            '--max-kbps 10000 ' +
-            '--max-burst-kbits 1400 ' +
-            '--egress ' + policy_name
+            'network qos rule create -f json '
+            '--type bandwidth-limit '
+            '--max-kbps 10000 '
+            '--max-burst-kbits 1400 '
+            '--egress %s' % policy_name
         ))
         raw_output = self.openstack(
-            'network qos rule delete ' +
-            policy_name + ' ' + rule['id'])
+            'network qos rule delete %s %s' %
+            (policy_name, rule['id']))
         self.assertEqual('', raw_output)
 
     def test_qos_rule_list(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule list -f json '
-            + self.QOS_POLICY_NAME))
+            'network qos rule list -f json %s' %
+            self.QOS_POLICY_NAME))
         self.assertIn(self.RULE_ID, [rule['ID'] for rule in cmd_output])
 
     def test_qos_rule_show(self):
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(self.RULE_ID, cmd_output['id'])
 
     def test_qos_rule_set(self):
-        self.openstack('network qos rule set --max-kbps 15000 ' +
-                       '--max-burst-kbits 1800 ' +
-                       '--ingress ' +
-                       self.QOS_POLICY_NAME + ' ' + self.RULE_ID)
+        self.openstack('network qos rule set --max-kbps 15000 '
+                       '--max-burst-kbits 1800 '
+                       '--ingress %s %s' %
+                       (self.QOS_POLICY_NAME, self.RULE_ID))
         cmd_output = json.loads(self.openstack(
-            'network qos rule show -f json ' +
-            self.QOS_POLICY_NAME + ' ' + self.RULE_ID))
+            'network qos rule show -f json %s %s' %
+            (self.QOS_POLICY_NAME, self.RULE_ID)))
         self.assertEqual(15000, cmd_output['max_kbps'])
         self.assertEqual(1800, cmd_output['max_burst_kbps'])
         self.assertEqual('ingress', cmd_output['direction'])
