@@ -13,12 +13,14 @@
 #
 
 import argparse
+
 import mock
 from mock import call
-
 from osc_lib import exceptions
 from osc_lib import utils
 
+from openstackclient.tests.unit.identity.v3 import fakes as project_fakes
+from openstackclient.tests.unit import utils as tests_utils
 from openstackclient.tests.unit.volume.v2 import fakes as volume_fakes
 from openstackclient.volume.v2 import volume_snapshot
 
@@ -32,6 +34,8 @@ class TestSnapshot(volume_fakes.TestVolume):
         self.snapshots_mock.reset_mock()
         self.volumes_mock = self.app.client_manager.volume.volumes
         self.volumes_mock.reset_mock()
+        self.project_mock = self.app.client_manager.identity.projects
+        self.project_mock.reset_mock()
 
 
 class TestSnapshotCreate(TestSnapshot):
@@ -104,27 +108,17 @@ class TestSnapshotCreate(TestSnapshot):
     def test_snapshot_create_without_name(self):
         arglist = [
             "--volume", self.new_snapshot.volume_id,
-            "--description", self.new_snapshot.description,
-            "--force"
         ]
         verifylist = [
             ("volume", self.new_snapshot.volume_id),
-            ("description", self.new_snapshot.description),
-            ("force", True)
         ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        columns, data = self.cmd.take_action(parsed_args)
-
-        self.snapshots_mock.create.assert_called_with(
-            self.new_snapshot.volume_id,
-            force=True,
-            name=None,
-            description=self.new_snapshot.description,
-            metadata=None,
+        self.assertRaises(
+            tests_utils.ParserException,
+            self.check_parser,
+            self.cmd,
+            arglist,
+            verifylist,
         )
-        self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
 
     def test_snapshot_create_without_volume(self):
         arglist = [
@@ -153,17 +147,19 @@ class TestSnapshotCreate(TestSnapshot):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, data)
 
-    def test_snapshot_create_without_remote_source(self):
+    def test_snapshot_create_with_remote_source(self):
         arglist = [
             '--remote-source', 'source-name=test_source_name',
             '--remote-source', 'source-id=test_source_id',
             '--volume', self.new_snapshot.volume_id,
+            self.new_snapshot.name,
         ]
         ref_dict = {'source-name': 'test_source_name',
                     'source-id': 'test_source_id'}
         verifylist = [
             ('remote_source', ref_dict),
             ('volume', self.new_snapshot.volume_id),
+            ("snapshot_name", self.new_snapshot.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -172,7 +168,7 @@ class TestSnapshotCreate(TestSnapshot):
         self.snapshots_mock.manage.assert_called_with(
             volume_id=self.new_snapshot.volume_id,
             ref=ref_dict,
-            name=None,
+            name=self.new_snapshot.name,
             description=None,
             metadata=None,
         )
@@ -278,6 +274,7 @@ class TestSnapshotDelete(TestSnapshot):
 class TestSnapshotList(TestSnapshot):
 
     volume = volume_fakes.FakeVolume.create_one_volume()
+    project = project_fakes.FakeProject.create_one_project()
     snapshots = volume_fakes.FakeSnapshot.create_snapshots(
         attrs={'volume_id': volume.name}, count=3)
 
@@ -321,6 +318,7 @@ class TestSnapshotList(TestSnapshot):
 
         self.volumes_mock.list.return_value = [self.volume]
         self.volumes_mock.get.return_value = self.volume
+        self.project_mock.get.return_value = self.project
         self.snapshots_mock.list.return_value = self.snapshots
         # Get the command to test
         self.cmd = volume_snapshot.ListVolumeSnapshot(self.app, None)
@@ -341,6 +339,7 @@ class TestSnapshotList(TestSnapshot):
                 'all_tenants': False,
                 'name': None,
                 'status': None,
+                'project_id': None,
                 'volume_id': None
             }
         )
@@ -351,11 +350,13 @@ class TestSnapshotList(TestSnapshot):
         arglist = [
             "--long",
             "--limit", "2",
+            "--project", self.project.id,
             "--marker", self.snapshots[0].id,
         ]
         verifylist = [
             ("long", True),
             ("limit", 2),
+            ("project", self.project.id),
             ("marker", self.snapshots[0].id),
             ('all_projects', False),
         ]
@@ -367,7 +368,8 @@ class TestSnapshotList(TestSnapshot):
             limit=2,
             marker=self.snapshots[0].id,
             search_opts={
-                'all_tenants': False,
+                'all_tenants': True,
+                'project_id': self.project.id,
                 'name': None,
                 'status': None,
                 'volume_id': None
@@ -394,6 +396,7 @@ class TestSnapshotList(TestSnapshot):
                 'all_tenants': True,
                 'name': None,
                 'status': None,
+                'project_id': None,
                 'volume_id': None
             }
         )
@@ -419,6 +422,7 @@ class TestSnapshotList(TestSnapshot):
                 'all_tenants': False,
                 'name': self.snapshots[0].name,
                 'status': None,
+                'project_id': None,
                 'volume_id': None
             }
         )
@@ -444,6 +448,7 @@ class TestSnapshotList(TestSnapshot):
                 'all_tenants': False,
                 'name': None,
                 'status': self.snapshots[0].status,
+                'project_id': None,
                 'volume_id': None
             }
         )
@@ -469,6 +474,7 @@ class TestSnapshotList(TestSnapshot):
                 'all_tenants': False,
                 'name': None,
                 'status': None,
+                'project_id': None,
                 'volume_id': self.volume.id
             }
         )
@@ -499,7 +505,23 @@ class TestSnapshotSet(TestSnapshot):
         # Get the command object to mock
         self.cmd = volume_snapshot.SetVolumeSnapshot(self.app, None)
 
-    def test_snapshot_set(self):
+    def test_snapshot_set_no_option(self):
+        arglist = [
+            self.snapshot.id,
+        ]
+        verifylist = [
+            ("snapshot", self.snapshot.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.snapshots_mock.get.assert_called_once_with(parsed_args.snapshot)
+        self.assertNotCalled(self.snapshots_mock.reset_state)
+        self.assertNotCalled(self.snapshots_mock.update)
+        self.assertNotCalled(self.snapshots_mock.set_metadata)
+        self.assertIsNone(result)
+
+    def test_snapshot_set_name_and_property(self):
         arglist = [
             "--name", "new_snapshot",
             "--property", "x=y",
@@ -526,6 +548,51 @@ class TestSnapshotSet(TestSnapshot):
         )
         self.assertIsNone(result)
 
+    def test_snapshot_set_with_no_property(self):
+        arglist = [
+            "--no-property",
+            self.snapshot.id,
+        ]
+        verifylist = [
+            ("no_property", True),
+            ("snapshot", self.snapshot.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.snapshots_mock.get.assert_called_once_with(parsed_args.snapshot)
+        self.assertNotCalled(self.snapshots_mock.reset_state)
+        self.assertNotCalled(self.snapshots_mock.update)
+        self.assertNotCalled(self.snapshots_mock.set_metadata)
+        self.snapshots_mock.delete_metadata.assert_called_with(
+            self.snapshot.id, ["foo"]
+        )
+        self.assertIsNone(result)
+
+    def test_snapshot_set_with_no_property_and_property(self):
+        arglist = [
+            "--no-property",
+            "--property", "foo_1=bar_1",
+            self.snapshot.id,
+        ]
+        verifylist = [
+            ("no_property", True),
+            ("property", {"foo_1": "bar_1"}),
+            ("snapshot", self.snapshot.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.snapshots_mock.get.assert_called_once_with(parsed_args.snapshot)
+        self.assertNotCalled(self.snapshots_mock.reset_state)
+        self.assertNotCalled(self.snapshots_mock.update)
+        self.snapshots_mock.delete_metadata.assert_called_with(
+            self.snapshot.id, ["foo"]
+        )
+        self.snapshots_mock.set_metadata.assert_called_once_with(
+            self.snapshot.id, {"foo_1": "bar_1"})
+        self.assertIsNone(result)
+
     def test_snapshot_set_state_to_error(self):
         arglist = [
             "--state", "error",
