@@ -16,88 +16,131 @@
 """Volume v1 Volume action implementations"""
 
 import argparse
+import logging
+
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
 import six
 
-from openstackclient.common import command
-from openstackclient.common import parseractions
-from openstackclient.common import utils
+from openstackclient.i18n import _
+
+
+LOG = logging.getLogger(__name__)
+
+
+def _check_size_arg(args):
+    """Check whether --size option is required or not.
+
+    Require size parameter only in case when snapshot or source
+    volume is not specified.
+    """
+
+    if ((args.snapshot or args.source)
+            is None and args.size is None):
+        msg = _("--size is a required option if snapshot "
+                "or source volume is not specified.")
+        raise exceptions.CommandError(msg)
 
 
 class CreateVolume(command.ShowOne):
-    """Create new volume"""
+    _description = _("Create new volume")
 
     def get_parser(self, prog_name):
         parser = super(CreateVolume, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<name>',
-            help='Volume name',
+            help=_('Volume name'),
         )
         parser.add_argument(
             '--size',
             metavar='<size>',
-            required=True,
             type=int,
-            help='Volume size in GB',
+            help=_("Volume size in GB (Required unless --snapshot or "
+                   "--source is specified)"),
         )
         parser.add_argument(
             '--type',
             metavar='<volume-type>',
-            help="Set the type of volume",
+            help=_("Set the type of volume"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             '--image',
             metavar='<image>',
-            help='Use <image> as source of volume (name or ID)',
+            help=_('Use <image> as source of volume (name or ID)'),
         )
-        snapshot_group = parser.add_mutually_exclusive_group()
-        snapshot_group.add_argument(
+        source_group.add_argument(
             '--snapshot',
             metavar='<snapshot>',
-            help='Use <snapshot> as source of volume (name or ID)',
+            help=_('Use <snapshot> as source of volume (name or ID)'),
         )
-        snapshot_group.add_argument(
+        source_group.add_argument(
             '--snapshot-id',
             metavar='<snapshot-id>',
             help=argparse.SUPPRESS,
         )
-        parser.add_argument(
+        source_group.add_argument(
             '--source',
             metavar='<volume>',
-            help='Volume to clone (name or ID)',
+            help=_('Volume to clone (name or ID)'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='Volume description',
+            help=_('Volume description'),
         )
         parser.add_argument(
             '--user',
             metavar='<user>',
-            help='Specify an alternate user (name or ID)',
+            help=_('Specify an alternate user (name or ID)'),
         )
         parser.add_argument(
             '--project',
             metavar='<project>',
-            help='Specify an alternate project (name or ID)',
+            help=_('Specify an alternate project (name or ID)'),
         )
         parser.add_argument(
             '--availability-zone',
             metavar='<availability-zone>',
-            help='Create volume in <availability-zone>',
+            help=_('Create volume in <availability-zone>'),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Set a property on this volume '
-                 '(repeat option to set multiple properties)',
+            help=_('Set a property on this volume '
+                   '(repeat option to set multiple properties)'),
+        )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable (default)")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode (default)")
         )
 
         return parser
 
     def take_action(self, parsed_args):
-
+        _check_size_arg(parsed_args)
         identity_client = self.app.client_manager.identity
         image_client = self.app.client_manager.image
         volume_client = self.app.client_manager.volume
@@ -145,6 +188,22 @@ class CreateVolume(command.ShowOne):
             parsed_args.property,
             image,
         )
+
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+
         # Map 'metadata' column to 'properties'
         volume._info.update(
             {
@@ -152,12 +211,18 @@ class CreateVolume(command.ShowOne):
                 'type': volume._info.pop('volume_type'),
             },
         )
+        # Replace "display_name" by "name", keep consistent in v1 and v2
+        if 'display_name' in volume._info:
+            volume._info.update({'name': volume._info.pop('display_name')})
+        volume_info = utils.backward_compat_col_showone(
+            volume._info, parsed_args.columns, {'display_name': 'name'}
+        )
 
-        return zip(*sorted(six.iteritems(volume._info)))
+        return zip(*sorted(six.iteritems(volume_info)))
 
 
 class DeleteVolume(command.Command):
-    """Delete volume(s)"""
+    _description = _("Delete volume(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteVolume, self).get_parser(prog_name)
@@ -165,55 +230,75 @@ class DeleteVolume(command.Command):
             'volumes',
             metavar='<volume>',
             nargs="+",
-            help='Volume(s) to delete (name or ID)',
+            help=_('Volume(s) to delete (name or ID)'),
         )
         parser.add_argument(
             '--force',
-            dest='force',
             action='store_true',
             default=False,
-            help='Attempt forced removal of volume(s), regardless of state '
-                 '(defaults to False)',
+            help=_('Attempt forced removal of volume(s), regardless of state '
+                   '(defaults to False)'),
         )
         return parser
 
     def take_action(self, parsed_args):
         volume_client = self.app.client_manager.volume
-        for volume in parsed_args.volumes:
-            volume_obj = utils.find_resource(
-                volume_client.volumes, volume)
-            if parsed_args.force:
-                volume_client.volumes.force_delete(volume_obj.id)
-            else:
-                volume_client.volumes.delete(volume_obj.id)
+        result = 0
+
+        for i in parsed_args.volumes:
+            try:
+                volume_obj = utils.find_resource(
+                    volume_client.volumes, i)
+                if parsed_args.force:
+                    volume_client.volumes.force_delete(volume_obj.id)
+                else:
+                    volume_client.volumes.delete(volume_obj.id)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete volume with "
+                            "name or ID '%(volume)s': %(e)s"),
+                          {'volume': i, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.volumes)
+            msg = (_("%(result)s of %(total)s volumes failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListVolume(command.Lister):
-    """List volumes"""
+    _description = _("List volumes")
 
     def get_parser(self, prog_name):
         parser = super(ListVolume, self).get_parser(prog_name)
         parser.add_argument(
             '--name',
             metavar='<name>',
-            help='Filter results by volume name',
+            help=_('Filter results by volume name'),
         )
         parser.add_argument(
             '--status',
             metavar='<status>',
-            help='Filter results by status',
+            help=_('Filter results by status'),
         )
         parser.add_argument(
             '--all-projects',
             action='store_true',
             default=False,
-            help='Include all projects (admin only)',
+            help=_('Include all projects (admin only)'),
         )
         parser.add_argument(
             '--long',
             action='store_true',
             default=False,
-            help='List additional fields in output',
+            help=_('List additional fields in output'),
+        )
+        parser.add_argument(
+            '--limit',
+            type=int,
+            action=parseractions.NonNegativeAction,
+            metavar='<num-volumes>',
+            help=_('Maximum number of volumes to display'),
         )
         return parser
 
@@ -251,7 +336,7 @@ class ListVolume(command.Lister):
             )
             column_headers = (
                 'ID',
-                'Display Name',
+                'Name',
                 'Status',
                 'Size',
                 'Type',
@@ -269,7 +354,7 @@ class ListVolume(command.Lister):
             )
             column_headers = (
                 'ID',
-                'Display Name',
+                'Name',
                 'Status',
                 'Size',
                 'Attached to',
@@ -290,7 +375,12 @@ class ListVolume(command.Lister):
             'status': parsed_args.status,
         }
 
-        data = volume_client.volumes.list(search_opts=search_opts)
+        data = volume_client.volumes.list(
+            search_opts=search_opts,
+            limit=parsed_args.limit,
+        )
+        column_headers = utils.backward_compat_col_lister(
+            column_headers, parsed_args.columns, {'Display Name': 'Name'})
 
         return (column_headers,
                 (utils.get_item_properties(
@@ -300,38 +390,100 @@ class ListVolume(command.Lister):
                 ) for s in data))
 
 
+class MigrateVolume(command.Command):
+    _description = _("Migrate volume to a new host")
+
+    def get_parser(self, prog_name):
+        parser = super(MigrateVolume, self).get_parser(prog_name)
+        parser.add_argument(
+            'volume',
+            metavar="<volume>",
+            help=_("Volume to migrate (name or ID)")
+        )
+        parser.add_argument(
+            '--host',
+            metavar="<host>",
+            required=True,
+            help=_("Destination host (takes the form: host@backend-name#pool)")
+        )
+        parser.add_argument(
+            '--force-host-copy',
+            action="store_true",
+            help=_("Enable generic host-based force-migration, "
+                   "which bypasses driver optimizations")
+        )
+        return parser
+
+    def take_action(self, parsed_args):
+        volume_client = self.app.client_manager.volume
+        volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
+        volume_client.volumes.migrate_volume(volume.id, parsed_args.host,
+                                             parsed_args.force_host_copy,)
+
+
 class SetVolume(command.Command):
-    """Set volume properties"""
+    _description = _("Set volume properties")
 
     def get_parser(self, prog_name):
         parser = super(SetVolume, self).get_parser(prog_name)
         parser.add_argument(
             'volume',
             metavar='<volume>',
-            help='Volume to modify (name or ID)',
+            help=_('Volume to modify (name or ID)'),
         )
         parser.add_argument(
             '--name',
             metavar='<name>',
-            help='New volume name',
+            help=_('New volume name'),
         )
         parser.add_argument(
             '--description',
             metavar='<description>',
-            help='New volume description',
+            help=_('New volume description'),
         )
         parser.add_argument(
             '--size',
             metavar='<size>',
             type=int,
-            help='Extend volume size in GB',
+            help=_('Extend volume size in GB'),
+        )
+        parser.add_argument(
+            "--no-property",
+            dest="no_property",
+            action="store_true",
+            help=_("Remove all properties from <volume> "
+                   "(specify both --no-property and --property to "
+                   "remove the current properties before setting "
+                   "new properties.)"),
         )
         parser.add_argument(
             '--property',
             metavar='<key=value>',
             action=parseractions.KeyValueAction,
-            help='Set a property on this volume '
-                 '(repeat option to set multiple properties)',
+            help=_('Set a property on this volume '
+                   '(repeat option to set multiple properties)'),
+        )
+        bootable_group = parser.add_mutually_exclusive_group()
+        bootable_group.add_argument(
+            "--bootable",
+            action="store_true",
+            help=_("Mark volume as bootable")
+        )
+        bootable_group.add_argument(
+            "--non-bootable",
+            action="store_true",
+            help=_("Mark volume as non-bootable")
+        )
+        readonly_group = parser.add_mutually_exclusive_group()
+        readonly_group.add_argument(
+            "--read-only",
+            action="store_true",
+            help=_("Set volume to read-only access mode")
+        )
+        readonly_group.add_argument(
+            "--read-write",
+            action="store_true",
+            help=_("Set volume to read-write access mode")
         )
         return parser
 
@@ -339,42 +491,81 @@ class SetVolume(command.Command):
         volume_client = self.app.client_manager.volume
         volume = utils.find_resource(volume_client.volumes, parsed_args.volume)
 
+        result = 0
         if parsed_args.size:
-            if volume.status != 'available':
-                self.app.log.error("Volume is in %s state, it must be "
-                                   "available before size can be extended" %
-                                   volume.status)
-                return
-            if parsed_args.size <= volume.size:
-                self.app.log.error("New size must be greater than %s GB" %
-                                   volume.size)
-                return
-            volume_client.volumes.extend(volume.id, parsed_args.size)
+            try:
+                if volume.status != 'available':
+                    msg = (_("Volume is in %s state, it must be available "
+                           "before size can be extended") % volume.status)
+                    raise exceptions.CommandError(msg)
+                if parsed_args.size <= volume.size:
+                    msg = (_("New size must be greater than %s GB")
+                           % volume.size)
+                    raise exceptions.CommandError(msg)
+                volume_client.volumes.extend(volume.id, parsed_args.size)
+            except Exception as e:
+                LOG.error(_("Failed to set volume size: %s"), e)
+                result += 1
+
+        if parsed_args.no_property:
+            try:
+                volume_client.volumes.delete_metadata(
+                    volume.id, volume.metadata.keys())
+            except Exception as e:
+                LOG.error(_("Failed to clean volume properties: %s"), e)
+                result += 1
 
         if parsed_args.property:
-            volume_client.volumes.set_metadata(volume.id, parsed_args.property)
-
+            try:
+                volume_client.volumes.set_metadata(
+                    volume.id,
+                    parsed_args.property)
+            except Exception as e:
+                LOG.error(_("Failed to set volume property: %s"), e)
+                result += 1
+        if parsed_args.bootable or parsed_args.non_bootable:
+            try:
+                volume_client.volumes.set_bootable(
+                    volume.id, parsed_args.bootable)
+            except Exception as e:
+                LOG.error(_("Failed to set volume bootable property: %s"), e)
+                result += 1
+        if parsed_args.read_only or parsed_args.read_write:
+            try:
+                volume_client.volumes.update_readonly_flag(
+                    volume.id,
+                    parsed_args.read_only)
+            except Exception as e:
+                LOG.error(_("Failed to set volume read-only access "
+                            "mode flag: %s"), e)
+                result += 1
         kwargs = {}
         if parsed_args.name:
             kwargs['display_name'] = parsed_args.name
         if parsed_args.description:
             kwargs['display_description'] = parsed_args.description
         if kwargs:
-            volume_client.volumes.update(volume.id, **kwargs)
+            try:
+                volume_client.volumes.update(volume.id, **kwargs)
+            except Exception as e:
+                LOG.error(_("Failed to update volume display name "
+                          "or display description: %s"), e)
+                result += 1
 
-        if not kwargs and not parsed_args.property and not parsed_args.size:
-            self.app.log.error("No changes requested\n")
+        if result > 0:
+            raise exceptions.CommandError(_("One or more of the "
+                                          "set operations failed"))
 
 
 class ShowVolume(command.ShowOne):
-    """Show volume details"""
+    _description = _("Show volume details")
 
     def get_parser(self, prog_name):
         parser = super(ShowVolume, self).get_parser(prog_name)
         parser.add_argument(
             'volume',
             metavar='<volume>',
-            help='Volume to display (name or ID)',
+            help=_('Volume to display (name or ID)'),
         )
         return parser
 
@@ -393,27 +584,33 @@ class ShowVolume(command.ShowOne):
                 {'project_id': volume._info.pop(
                     'os-vol-tenant-attr:tenant_id')}
             )
-        return zip(*sorted(six.iteritems(volume._info)))
+        # Replace "display_name" by "name", keep consistent in v1 and v2
+        if 'display_name' in volume._info:
+            volume._info.update({'name': volume._info.pop('display_name')})
+
+        volume_info = utils.backward_compat_col_showone(
+            volume._info, parsed_args.columns, {'display_name': 'name'}
+        )
+
+        return zip(*sorted(six.iteritems(volume_info)))
 
 
 class UnsetVolume(command.Command):
-    """Unset volume properties"""
+    _description = _("Unset volume properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetVolume, self).get_parser(prog_name)
         parser.add_argument(
             'volume',
             metavar='<volume>',
-            help='Volume to modify (name or ID)',
+            help=_('Volume to modify (name or ID)'),
         )
         parser.add_argument(
             '--property',
             metavar='<key>',
             action='append',
-            default=[],
-            help='Remove a property from volume '
-                 '(repeat option to remove multiple properties)',
-            required=True,
+            help=_('Remove a property from volume '
+                   '(repeat option to remove multiple properties)'),
         )
         return parser
 
@@ -427,5 +624,3 @@ class UnsetVolume(command.Command):
                 volume.id,
                 parsed_args.property,
             )
-        else:
-            self.app.log.error("No changes requested\n")
