@@ -38,8 +38,7 @@ class CreateVolumeSnapshot(command.ShowOne):
         parser.add_argument(
             "snapshot_name",
             metavar="<snapshot-name>",
-            nargs="?",
-            help=_("Name of the new snapshot (default to None)")
+            help=_("Name of the new snapshot"),
         )
         parser.add_argument(
             "--volume",
@@ -180,14 +179,14 @@ class ListVolumeSnapshot(command.Lister):
         )
         parser.add_argument(
             '--marker',
-            metavar='<marker>',
+            metavar='<volume-snapshot>',
             help=_('The last snapshot ID of the previous page'),
         )
         parser.add_argument(
             '--limit',
             type=int,
             action=parseractions.NonNegativeAction,
-            metavar='<limit>',
+            metavar='<num-snapshots>',
             help=_('Maximum number of snapshots to display'),
         )
         parser.add_argument(
